package com.huiju.estateagents.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.JWTUtils;
import com.huiju.estateagents.entity.TaPerson;
import com.huiju.estateagents.entity.TaPersonVisitRecord;
import com.huiju.estateagents.service.ITaPersonService;
import com.huiju.estateagents.service.ITaPersonVisitRecordService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;


/**
 * <p>
    *   前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-07-23
 */
@RestController
@RequestMapping("/")
public class TaPersonVisitRecordController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaPersonVisitRecordController.class);

    @Autowired
    public ITaPersonVisitRecordService iTaPersonVisitRecordService;
    @Autowired
    public ITaPersonService taPersonService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taPersonVisitRecord",method= RequestMethod.GET)
    public ResponseBean taPersonVisitRecordList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                                @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaPersonVisitRecord> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaPersonVisitRecord> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaPersonVisitRecord> result = iTaPersonVisitRecordService.page(pg, queryWrapper);
            responseBean.success(result);
        }catch (Exception e){
            logger.error("taPersonVisitRecordList -=- {}",e.toString());
            responseBean.error(e.getMessage(),ResponseBean.ERROR_UNAVAILABLE);
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taPersonVisitRecord 实体对象
     * @return
     */
    @RequestMapping(value="/wx/taPersonVisitRecord",method= RequestMethod.POST)
    public ResponseBean taPersonVisitRecordAdd(@RequestBody TaPersonVisitRecord taPersonVisitRecord, HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        String openid = JWTUtils.getSubject(request);
        List<TaPerson> taPersons = taPersonService.getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }
        TaPerson person = taPersons.get(0);
        taPersonVisitRecord.setPersonId(person.getPersonId());
        taPersonVisitRecord.setVisitTime(LocalDateTime.now());
        try {
            if (iTaPersonVisitRecordService.save(taPersonVisitRecord)){
                responseBean.success(taPersonVisitRecord);
            }else {
                responseBean.error("保存失败",ResponseBean.ERROR_UNAVAILABLE);
            }
        }catch (Exception e){
            logger.error("taPersonVisitRecordAdd -=- {}",e.toString());
            responseBean.error("系统异常"+e.getMessage(),ResponseBean.ERROR_UNAVAILABLE);
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taPersonVisitRecord/{id}", method= RequestMethod.DELETE)
    public ResponseBean taPersonVisitRecordDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaPersonVisitRecordService.removeById(id)){
                responseBean.success("success");
            }else {
                responseBean.error("删除失败",ResponseBean.ERROR_UNAVAILABLE);
            }
        }catch (Exception e){
            logger.error("taPersonVisitRecordDelete -=- {}",e.toString());
            responseBean.error("保存失败"+e.getMessage(),ResponseBean.ERROR_UNAVAILABLE);
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taPersonVisitRecord 实体对象
     * @return
     */
    @RequestMapping(value="/wx/taPersonVisitRecord/{id}",method= RequestMethod.PUT)
    public ResponseBean taPersonVisitRecordUpdate(@PathVariable Integer id,
                                        @RequestBody TaPersonVisitRecord taPersonVisitRecord){
        ResponseBean responseBean = new ResponseBean();
        taPersonVisitRecord.setLeaveTime(LocalDateTime.now());
        taPersonVisitRecord.setRecordId(id);
        Long duation = taPersonVisitRecord.getLeaveTime().atZone(ZoneOffset.UTC).toEpochSecond()-taPersonVisitRecord.getVisitTime().atZone(ZoneOffset.UTC).toEpochSecond();
        taPersonVisitRecord.setVisitDuration(duation.intValue());
        try {
            if (iTaPersonVisitRecordService.updateById(taPersonVisitRecord)){
                responseBean.success(taPersonVisitRecord);
            }else {
                responseBean.error("更新失败",ResponseBean.ERROR_UNAVAILABLE);
            }
        }catch (Exception e){
            logger.error("taPersonVisitRecordUpdate -=- {}",e.toString());
            responseBean.error("更新失败"+e.getMessage(),ResponseBean.ERROR_UNAVAILABLE);
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taPersonVisitRecord/{id}",method= RequestMethod.GET)
    public ResponseBean taPersonVisitRecordGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.success(iTaPersonVisitRecordService.getById(id));
        }catch (Exception e){
            logger.error("taPersonVisitRecordDelete -=- {}",e.toString());
            responseBean.error(e.getMessage(),ResponseBean.ERROR_UNAVAILABLE);
        }
        return responseBean;
    }
}
